cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     module GRIMM_2x
c                       V. Lesur 30/09/2009
c
c    ALL THE SUBROUTINES IN THIS FILE SHOULD NOT BE DISTRIBUTED OR 
c    USED FOR COMMERCIAL PURPOSE. WE DON'T OFFER ANY GARANTY REGARDING 
c    THE ACCURACY OF THE CODE OUTPUTS.
c
c     In this module are defined the necessary variables and 
c     subroutines for estimating the values of the GRIMM-2x model 
c     truncated to SH 16 (see: Lesur et al. 2009)
c
c     SLATEC SUBROUTINES ARE REQUIRED. THEY ARE GIVEN IN THE FILE:
c     extract_slatec.f
c     THEY CAN BE INDEPENDENTLY DOWNLOADED FROM:
c     http://www.netlib.org/slatec/src/
c
c  Number of parameters for GRIMM-2 is:
c  ilg*(ilg+2)*itg                    internal spline model 
c
c    Global data:
c       ryg : real*8  : reference year for the model
c
c       ilg : integer : time varying max SH degree value 
c       itg : integer : time parameter for SH degree up to ilg
c
c       isog: integer : spline order
c       nkg : integer : number of time knots for splines
c       tkg : real*8  : array of spline knot positions 
c
c       nomg: charcater*100 : path & name of GRIMM coeff. file
c       yong: cahracter*1 : Indicate if the model is defined (y)
c                           or not (n)
c
c       nbtkg: integer : max number of spline nodes
c
c    Main Subroutines:
c      init_grimm_2x.f      set and initialise global variables
c      forward_grimm_2x.f   driver for the GRIMM-2x forward modelling
c      sub_grimm_2x.f       model subroutine
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
        module grimm_2x
c
        implicit none
c
        integer nbtkg
        parameter (nbtkg=22)
c
        character nomg*100,yong
c
        integer ilg,itg,isog
        real*8 ryg
c
        integer nkg
        real*8 tkg(nbtkg)
c
        contains
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c    subroutine init_grimm_2x
c
c       V. Lesur  30/09/2009
c
c  Set initial values for global data and model definition
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
        subroutine init_grimm_2x(nb,bc)
c
        implicit none
c
        integer nb,i,j,iw
        real*8 bc(*)
        real*8, allocatable :: dw(:,:)
c
c   path to GRIMM model file
        nomg='./grimm_2x.cof'
c
c   model not read yet
        yong='n'
c
c   model reference year
        call dy2mjd(2005.0d0,ryg)
c
c   core field max SH degree
        ilg=16
c
c   splines order
        isog=6
c
c   spline knots
        nkg=22
        itg=nkg-isog
        tkg(1)=0.d0
        tkg(2)=0.d0
        tkg(3)=0.d0
        tkg(4)=0.d0
        tkg(5)=0.d0
        tkg(6)=0.d0
        tkg(7)=366.d0
        tkg(8)=731.d0
        tkg(9)=1096.d0
        tkg(10)=1461.d0
        tkg(11)=1827.d0
        tkg(12)=2192.d0
        tkg(13)=2557.d0
        tkg(14)=2922.d0
        tkg(15)=3288.d0
        tkg(16)=3653.d0
        tkg(17)=4018.d0
        tkg(18)=4018.d0
        tkg(19)=4018.d0
        tkg(20)=4018.d0
        tkg(21)=4018.d0
        tkg(22)=4018.d0
        do i=1,nkg
          tkg(i)=(tkg(i)-ryg)/365.25d0
        enddo
c
c   model dim & bc set to 0
        nb=ilg*(ilg+2)*itg
        bc(1:nb)=0.0d0
c
        return
        end subroutine init_grimm_2x
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c    subroutine forward_grimm_2x
c                                     v.lesur  30-09-2009
c
c    from a position and time computes the X (NORTH),Y (EAST),Z (DOWN) 
c    and F (INTENSITY) components of the magnetic field as given by the
c    GRIMM_2. 
c
c     The model time span is from MJD=366 (1st Jan 2001) 
c                              to MJD=3500 (1st Aug 2009)
c
c    All inpouts and outputs are in GEOCENTRIC
c
c    INPUTS:
c        nb         INTEGER    model dimm
c        bc(*)      REAL*8     array of GAUSS coef. (dim=nb)
c        clt        REAL*8     Latitude geocentric
c        dln        REAL*8     Longitude geocentric
c        rad        REAL*8     Radius geocentric
c        dt         REAL*8     decimal year
c    OUPUTS:
c        dx         REAL*8     North component in geocentric
c        dy         REAL*8     East component in geocentric
c        dz         REAL*8     Down component in geocentric
c        df         REAL*8     total intensity
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
        subroutine forward_grimm_2x(nb,bc,clt,dln,rad,dt,dx,dy,dz,df)
c
        implicit none
c
        integer nb
        real*8 bc(*),clt,dln,rad,dt,dx,dy,dz,df
c
        character cc
        integer i,j
        real*8 dd,djd,bp(4)
        real*8, allocatable :: be(:)
c
        allocate (be(1:nb))
c
c  Reading model if necessary
        if(yong.eq.'n')then
          open(10,file=nomg,status='old')
            read(10,*)cc
            read(10,*)cc
            do i=1,nb
              read(10,*)cc,j,j,bc(i)
            enddo
          close(10)
          yong='y'
        endif
c
c  Finding times in MJD 
        call dy2mjd(dt,djd)
c
c  Building bp vector
        bp(1)=clt
        bp(2)=dln
        bp(3)=rad
        bp(4)=djd
c
c  Calculating X components
        call sub_grimm_2x('i',1,nb,bc,bp,be)
        dx=0
        do i=1,nb
          dx=dx+be(i)*bc(i)
        enddo
c
c  Calculating Y components
        call sub_grimm_2x('i',2,nb,bc,bp,be)
        dy=0
        do i=1,nb
          dy=dy+be(i)*bc(i)
        enddo
c
c  Calculating Z components
        call sub_grimm_2x('i',3,nb,bc,bp,be)
        dz=0
        do i=1,nb
          dz=dz+be(i)*bc(i)
        enddo
c
c  Calculating F data
        df=dsqrt(dx**2+dy**2+dz**2)
c
        deallocate (be)
c
        return
        end subroutine forward_grimm_2x
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c    subroutine sub_grimm_2x
c
c       V. Lesur  30/09/2009
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
        subroutine sub_grimm_2x(iof,nub,nb,bc,bp,be)
c
        implicit none
c
        character iof
        integer nub,nb
        real*8 bp(*),be(*),bc(*)
c
        integer i,ik,nui
        real*8 dt
c
c  Set values
        be(1:nb)=0.0d0
        dt=(bp(4)-ryg)/365.25d0
        if(dt.le.tkg(1).or.dt.ge.tkg(nkg)) then
          write(*,*)'ERROR in sub_grimm_2x'
          write(*,*)'date outside model time span'
          stop
        endif
c
c  defining pointer for model
        nui=1
c
c  Finding tkg interval
        ik=2
        do while (dt.ge.tkg(ik).and.ik.lt.nkg)
          ik=ik+1
        enddo
        ik=ik-1
c
c  calculate equation of condition depending on components
c
c X (North)
        if(nub.eq.1)then
          call Xsph_bcore_spl2(isog,nkg,tkg,ik,ryg,1,ilg,bp,be(nui))
c
c Y (East)
        elseif(nub.eq.2)then
          call Ysph_bcore_spl2(isog,nkg,tkg,ik,ryg,1,ilg,bp,be(nui))
c
c Z (down)
        elseif(nub.eq.3)then
          call Zsph_bcore_spl2(isog,nkg,tkg,ik,ryg,1,ilg,bp,be(nui))
c
c Error
        else
          write(*,*)"ERROR in sub_grimm_2x.f"
          stop
        endif
c
        if(iof.eq.'f')be(1:nb)=be(1:nb)*bc(1:nb)
c
        return
        end subroutine  sub_grimm_2x
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c
c   FROM HERE DOWN: SERVICE SUBROUTINES EXTRACTED FROM MY OWN FORTRAN LIB
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c	Subroutine Xsph_bcore_spl2
c		Vincent Lesur 29/08/2005
c
c      modified 24/01/2007 to remove common and modify input list
c      modified 24/05/2007 to input spline order
c
c       For a all spherical harmonique up to degree nli 
c       and a  colatitude, longitude, radius and time computes 
c       the value of the X component basis  (X horizontal North) 
c       in GEO system of coordinates
c 
c       This basis is the derivative along the (-colatitude) of a potential 
c       defined on the sphere of radius "ra"  
c       (See Foundations of Geomagnetism, Backus 1996 page 110 and 125)
c       Time dependence of the potential is introduce via Bspline basis 
c       of order io 
c
c       Spherical harmonics are set by increasing degree "l", for
c       a given degree by increasing order "m" and for
c       each lm paires by spline basis values:
c       nu=(l*l+2*(m-1))*nti+it   FOR cosine terms 
c       nu=(l*l+2*(m-1)+1)*nti+it For sine terms
c
c       up to degree L there are L*(L+2)*nti coeffs
c
c       colat,long,radius given in degree,degree,km
c
c       input:
c         io            spline order
c         nk            number of spline knots
c         tk            spline knots positions
c         ik            spline knot tk(ik)=< dt <tk(ik+1)
c         ry            reference year
c         nlis/e        starting/ending degree internal
c         pos           positions in space and time (colat,long,radius,JD)
c
c       output:
c         be		Base funtion value
c       
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
	subroutine Xsph_bcore_spl2(io,nk,tk,ik,ry,nlis,nlie,pos,be)
c
        implicit none
c
        integer io,nk,ik,nlis,nlie,iw
        real*8 tk(*),ry,pos(*),be(*)
        integer nu,nus,nti,il,im,i,it
        real*8 rc,rs,ra,rw,ds,dc,dt
        real*8 tw(io)
c       real*8 dcosd_vl,dsind_vl
        real*8, allocatable :: ddlf(:),dw(:)
c
c       external dcosd_vl,dsind_vl
c
        nti=nk-io
        iw=int(float(io+1)*float(io+2)/2.)
        allocate(ddlf(1:nlie+1),dw(1:iw))
c
        ra=3485.0d0
        dt=(pos(4)-ry)/365.25d0
        rc=dcosd_vl(pos(1))
        rs=dsind_vl(pos(1))
        nus=(nlis*nlis-1)*nti
c
c       Define splines
        call DBSPVD(tk,io,1,dt,ik,io,tw,dw)
c
c   im=0 
        im=0
        call mkdlf_F(im,nlie,rs,rc,ddlf)
        do il=nlis,nlie
          nu=(il*il-1)*nti-nus
          rw=ddlf(il-im+1)*(ra/pos(3))**(il+2)
          do i=1,io
            it=ik-io+i
            be(nu+it)=rw*tw(i)
          enddo
        enddo
c   im.ne.0
        do im=1,nlie
          call mkdlf_F(im,nlie,rs,rc,ddlf)
          dc=dcosd_vl(im*pos(2))
          ds=dsind_vl(im*pos(2))
          do il=max0(im,nlis),nlie
            rw=ddlf(il-im+1)*(ra/pos(3))**(il+2)
            nu=(il*il+2*(im-1))*nti-nus
            do i=1,io
              it=ik-io+i
              be(nu+it)=rw*dc*tw(i)
              be(nu+nti+it)=rw*ds*tw(i)
            enddo
          enddo
        enddo
c
        deallocate(ddlf,dw)
        return
        end subroutine Xsph_bcore_spl2
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c	Subroutine Ysph_bcore_spl2
c		Vincent Lesur 29/08/2005
c
c      modified 24/01/2007 to remove common and modify input list
c      modified 24/05/2007 to input spline order
c
c       For a all spherical harmonique up to degree nl (nm,nl) 
c       and a  colatitude, longitude, radius and time compute 
c       the value of the Y component basis  (Y Positive East) 
c       in GEO system of coordinates
c
c       This basis is the derivative along longitude of an potential defined
c       on the sphere of radius "ra"
c       (See Foundations of Geomagnetism, Backus 1996 page 110 and 125)
c       Time dependence of the potential is introduce via Bspline basis
c       of order io
c
c       Spherical harmonics are set by increasing degree "l", for
c       a given degree by increasing order "m" and for
c       each lm paires by spline basis values:
c       nu=(l*l+2*(m-1))*nti+it   FOR cosine terms
c       nu=(l*l+2*(m-1)+1)*nti+it For sine terms
c
c       up to degree L there are L*(L+2)*nti coeffs
c
c       colat,long,radius given in degree,degree,km
c
c       input:
c         io            spline order
c         nk            number of spline knots
c         tk            spline knots positions
c         ik            spline knot tk(ik)=< dt <tk(ik+1)
c         ry            reference year
c         nlis/e        starting/ending degree internal
c         pos           positions in space and time (colat,long,radius,JD)
c
c       output:
c         be            Base funtion value
c       
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
	subroutine Ysph_bcore_spl2(io,nk,tk,ik,ry,nlis,nlie,pos,be)
c
        implicit none
c
        integer io,iw,nk,ik,nlis,nlie
        real*8 tk(*),ry,pos(*),be(*)
        integer nu,nus,nti,il,im,i,it
        real*8 rc,rs,ra,rw,ds,dc,dt
        real*8 tw(io)
c       real*8 dcosd_vl,dsind_vl
        real*8, allocatable :: dlf(:),dw(:)
c
c       external dcosd_vl,dsind_vl
c
        nti=nk-io
        iw=int(float(io+1)*float(io+2)/2.)
        allocate(dlf(1:nlie+1),dw(1:iw))
c
        ra=3485.0d0
        dt=(pos(4)-ry)/365.25d0
        rc=dcosd_vl(pos(1))
        rs=dsind_vl(pos(1))
        nus=(nlis*nlis-1)*nti
c
c       Define splines
        call DBSPVD(tk,io,1,dt,ik,io,tw,dw)
c
c   im=0
        im=0
        do il=nlis,nlie
          nu=(il*il-1)*nti-nus
          do i=1,io
            it=ik-io+i
            be(nu+it)=0.d0
          enddo
        enddo
c
c   im.ne.0
        do im=1,nlie
          call mklf_F(im,nlie,rs,rc,dlf)
          dc=-im*dsind_vl(im*pos(2))
          ds=im*dcosd_vl(im*pos(2))
          do il=max0(im,nlis),nlie
            rw=dlf(il-im+1)/rs
            rw=rw*(ra/pos(3))**(il+2)
            nu=(il*il+2*(im-1))*nti-nus
            do i=1,io
              it=ik-io+i
              be(nu+it)=-rw*dc*tw(i)
              be(nu+nti+it)=-rw*ds*tw(i)
            enddo
          enddo
        enddo
c
        deallocate(dlf,dw)
        return
        end subroutine Ysph_bcore_spl2
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c	Subroutine Zsph_bcore_spl2
c		Vincent Lesur 29/08/2005
c
c      modified 24/01/2007 to remove common and modify input list
c      modified 24/05/2007 to input spline order
c
c       For a all spherical harmonique up to degree nlie (nm,nl)
c       and a  colatitude, longitude, radius and time compute
c       the value of the Z component basis  (Z vertical DOWN) 
c       in GEO system of coordinates
c
c       This basis is the vertical derivative of an potential defined 
c       on the sphere of radius "ra"
c       (See Foundations of Geomagnetism, Backus 1996 page 110 and 125)
c       Time dependence of the potential is introduce via Bspline basis
c       of order io
c
c       Spherical harmonics are set by increasing degree "l", for
c       a given degree by increasing order "m" and for
c       each lm paires by spline basis values:
c       nu=(l*l+2*(m-1))*nti+it   FOR cosine terms
c       nu=(l*l+2*(m-1)+1)*nti+it For sine terms
c
c       up to degree L there are L*(L+2)*nti coeffs
c
c       colat,long,radius given in degree,degree,km
c
c       input:
c         io            spline order
c         nk            number of spline knots
c         tk            spline knots positions
c         ik            spline knot tk(ik)=< dt <tk(ik+1)
c         ry            reference year
c         nlis/e        starting/ending degree internal
c         pos           positions in space and time (colat,long,radius,JD)
c
c       output:
c         be            Base funtion value
c       
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
	subroutine Zsph_bcore_spl2(io,nk,tk,ik,ry,nlis,nlie,pos,be)
c
        implicit none
c
        integer io,iw,nk,ik,nlis,nlie
        real*8 tk(*),ry,pos(*),be(*)
        integer nu,nus,nti,il,im,i,it
        real*8 rc,rs,ra,rw,ds,dc,dt
        real*8 tw(io)
c       real*8 dcosd_vl,dsind_vl
        real*8, allocatable :: dlf(:),dw(:)
c
c       external dcosd_vl,dsind_vl
c
        nti=nk-io
        iw=int(float(io+1)*float(io+2)/2.)
        allocate(dlf(1:nlie+1),dw(1:iw))
c
        ra=3485.0d0
        dt=(pos(4)-ry)/365.25d0
        rc=dcosd_vl(pos(1))
        rs=dsind_vl(pos(1))
        nus=(nlis*nlis-1)*nti
c
c       Define splines
        call DBSPVD(tk,io,1,dt,ik,io,tw,dw)
c
c   im=0
        im=0
        call mklf_F(im,nlie,rs,rc,dlf)
        do il=nlis,nlie
          nu=(il*il-1)*nti-nus
          rw=-dlf(il-im+1)*dble(il+1)*(ra/pos(3))**(il+2)
          do i=1,io
            it=ik-io+i
            be(nu+it)=rw*tw(i)
          enddo
        enddo
c   im.ne.0
        do im=1,nlie
          call mklf_F(im,nlie,rs,rc,dlf)
          dc=dcosd_vl(im*pos(2))
          ds=dsind_vl(im*pos(2))
          do il=max0(im,nlis),nlie
            rw=dlf(il-im+1)*dble(il+1)*(ra/pos(3))**(il+2)
            nu=(il*il+2*(im-1))*nti-nus
            do i=1,io
              it=ik-io+i
              be(nu+it)=-rw*dc*tw(i)
              be(nu+nti+it)=-rw*ds*tw(i)
            enddo
          enddo
        enddo
c
        deallocate(dlf,dw)
        return
        end subroutine Zsph_bcore_spl2
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c	subroutine dy2mjd
c
c       V. Lesur 16/06/06
c
c tranform a date given in decimal year in Julian days from 00:00 
c on 01/01/2000
c
c subroutine called: IDAYSINYR
c
c input:
c         dy          decimal year (real*8)
c
c output:
c         djd         julian day (real*8)
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
        subroutine dy2mjd(dy,djd)
c
        implicit real*8 (a-h,o-z)
        implicit integer*4 (i-n)
c
        real*8 dy,djd
c
        integer iy,nd
        real*8 ddiy
c
c year and day number in year
        iy=idint(dy)
        ddiy=dble(IDAYSINYR(iy))
c
c number of days from 01/01/2000 at 00:00 to 01/01/iy at 00:00
        nd=0
        if(iy.ge.2000)then
          do i=2000,iy-1
            nd=nd+IDAYSINYR(i) 
          enddo
        else
          do i=iy,1999
             nd=nd+IDAYSINYR(i)
          enddo
          nd=-nd
        endif
c
c add days, hours, minutes and seconds 
        djd=dble(nd)+(dy-dble(iy))*ddiy
c
        return
        end  subroutine dy2mjd
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c 	subroutine mkdlf_F
c		V. Lesur 01/02/2002
c
c	Computes the derivative along theta of the legendre function
c       at a given order "m" from degree m to nl (nl > m).
c       Code based on the MKSSP subroutine writen by V.Lesur 9/17/97
c
c       CALLED: mklf_F.f
c
c       limitations of mklf_F apply
c
c       Fast version: tests reduced to minimum
c
c       input:
c         nm/nl		order and degree max
c         dc/ds         cos(colatitude)/sin(colatitude)
c	output:
c	  ddlf		derivative of legendre function from nm to nl
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
	subroutine mkdlf_F(nm,nl,ds,dc,ddlf)
c
        implicit none
c
c
        integer nm,nl,il,jl
        real*8 ds,dc,ddlf(*),d1,d2
c
        if (ds.eq.0.0d0) then
          if(nm.ne.1) then 
            do il=1,nl+1
              ddlf(il)=0.0d0
            enddo
          else
            ddlf(1)=-1.0d0
            ddlf(2)=-dsqrt(3.0d0) 
            do il=3,nl+1
              d1=(2*il-1)/dsqrt(dble(il*il-1))
              d2=dsqrt(dble((il-1)**2-1)/(il*il-1))
              ddlf(il)=d1*ddlf(il-1)-d2*ddlf(il-2)
            enddo
          endif
        else
c
          call mklf_F(nm,nl,ds,dc,ddlf)
c
          do il=nl,nm,-1
            jl=il-nm+1
            if(il-nm.gt.0)then
              d1=dsqrt(dble((il-nm)*(il+nm)))
              d2=dble(il) 
              ddlf(jl)=(d2*dc*ddlf(jl)-d1*ddlf(jl-1))/ds
            else
              ddlf(1)=nm*dc*ddlf(1)/ds
            endif
          enddo
        endif
c
99999   return
        end subroutine mkdlf_F
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c	subroutine mklf_F
c		V. Lesur 01/02/2001
c
c    Modified 23/02/04 : GOTO removed
c
c       computes the Schmidt normalised legendre functions dlf for 
c       a given order "m" from degree m to lm (lm > m). 
c       Fast version: minimium check, do not avoid underflows
c       
c       recurrence 3.7.28 Fundations of geomagetism, Backus 1996
c
c    inputs:
c	dc/ds           cos(colatitude)/sin(colatitude)
c       nm/lm           order and degree max 
c    outputs:
c       dlf             Legendre functions [dim min: max (2,lm-nm+1)]
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
        subroutine  mklf_F(nm,lm,ds,dc,dlf)
c
        implicit none
c
        integer nm,lm,ierr,il
        real*8 dlf(*),ds,dc,dnm,d1,d2,dbeta,dalpha
c
        real*8 dgamln
        external dgamln
c
c    start calculs
c
c     l=nm,  l=nm+1
c
        dnm=dble(nm)				!dble real for nm
        d1=dgamln(2*dnm+1.0d0,ierr)     	! d1=log(fact(2dnm))
        d2=dgamln(dnm+1.0d0,ierr)       	! d2=log(fact(dnm))
        if (ierr.ne.0) then
          write(*,*)'mklf_F: Cannot computes normalisation cst !'
          stop
        endif
c
        d2=0.5d0*d1-d2                  	!d2=sqrt(fact(2dnm))/fact(dnm)
        d2=d2-dnm*dlog(2.0d0)			!
        d2=dexp(d2)				!normalisation cst.
        if (nm.ne.0) d2=d2*dsqrt(2.0d0)		!special case  m=0
c
        d1=ds
        if(d1.ne.0.0d0) then
          d1=d1**nm
        else
          if(nm.eq.0)d1=1.d0
        endif
c
        dlf(1)=d1*d2				!leg. func. (m,m)
        dlf(2)=dlf(1)*dc*dsqrt(dble(2*nm+1))	!leg. func. (m+1,m)
c
c     l=nm+2....lm
c
        do il=2,lm-nm				!l=nm+il-1
c
          d1=dble((il-1)*(il+2*nm-1))		!(l-m)*(l+m)
          d2=dble((il)*(il+2*nm))		!(l-m+1)*(l+m+1)	
          dbeta=dsqrt(d1/d2)			! recurrence coeff.
          d1=dble(2*(il+nm)-1)			!2l+1
          dalpha=d1/dsqrt(d2)			!
c
          dlf(il+1)=dalpha*dlf(il)*dc		! 
     +                     -dbeta*dlf(il-1)	!leg. func. (nm+il-1,nm)
        enddo					
c
        return
        end subroutine  mklf_F
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c       function idaysinyr
c                              V. Lesur
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
        integer function idaysinyr(iy)
c
        implicit none
c
        integer iy
        integer ii
c
        ii=365
        if(mod(iy,4).eq.0)then
          ii=ii+1
          if(mod(iy,100).eq.0.and.mod(iy,400).ne.0) ii=ii-1
        endif
c
        idaysinyr=ii
c
        return
        end function idaysinyr
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c   degree_sincos functions
c
c   Calulate de cosinus and sinus of an angle given in degree
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         real*8 function dsind_vl(theta)
c
         implicit none
c
         real*8 theta,d2r
c
         d2r=4.d0*datan(1.d0)/180.d0
         dsind_vl=dsin(theta*d2r)
c
         return
         end function dsind_vl
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         real*8 function dcosd_vl(theta)
c
         implicit none
c
         real*8 theta,d2r
c
         d2r=4.d0*datan(1.d0)/180.d0
         dcosd_vl=dcos(theta*d2r)
c
         return
         end  function dcosd_vl
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
         end module grimm_2x
